/*
** ============================================================================
**
** FILE
**  main.c
**
** DESCRIPTION
**  This is the main file of the project.	    
**  HW platform SDB with C8051F93x Silabs MCU 
**
** CREATED
**  Silicon Laboratories Hungary Ltd
**
** COPYRIGHT
**  Copyright 2009 Silicon Laboratories, Inc.  
**	http://www.silabs.com
**
** ============================================================================
*/

								/* ======================================================== *
								 *									INCLUDE					*
							 	 * ======================================================== */


#include "C8051F930_defs.h"
#include "compiler_defs.h"
#include "S8051.h"
#include "radio.h"
#include "spi.h"
#include "user_setup.h"

#if DEGLITCH_METHOD == 1
#include "deglitch_1.h"
#endif

#if DEGLITCH_METHOD == 2
#include "deglitch_2.h"
#endif

#ifdef PACKET_HANDLER
#include "retime_phandler.h"
#endif

								/* ======================================================== *
								 *							Function PROTOTYPES				* 
								 * ======================================================== */
void MCU_Init(void);
void Deglitch_Init ( void );
void Deglitch_Start ( void );
void Deglitch_Stop ( void );


								/* ======================================================== *
								 *							Global VARIABLES				* 
								 * ======================================================== */


#ifdef PACKET_HANDLER
U8 DataBitBuffer [2][32];
U8 RetimedBitBuffer [40];
U8 pointer;
bit fBitBufferFull;
U8 fBufferSelect, fLastBufferSelect;
#endif

U32 RX_CLK_Clock_Rate;

/*------------------------------------------------------------------------*/
/*						GLOBAL variables								  */
/*------------------------------------------------------------------------*/

void main(void)
{


#ifdef PACKET_HANDLER

	U8 i, j, NoOfRetimedBits;
	U8 CheckSum;
	U16 SamplePerBitRatio;


	DeglitchedBitCnt = 1;
	fRetiming = 0;									
	fPacketHandler = 0;
	fSyncWordFound = 0;								
	fPacketReceived = 0;
	SyncWordBitCnt = 0;								
	PayloadBitCnt = 0;
	pointer = 0;
	NoOfRetimedBits = 0;
	BitValue = 0;
	fBufferSelect = 0;
	fLastBufferSelect = 0;


	for ( i = 0; i < PAYLOAD_BYTE_LENGTH; i++ )
		PayloadBuffer[i] = 0;

	for(i=0;i<32;i++)
		RetimedBitBuffer[i] = 0;


	// We have to use integer numbers because of the speed. If the sample per bit ratio is not a 
	// round number (for example 5.5 samples/bit) we can accumulate error when we compute how many real databits
	// were received without edge change (long run of 1's or 0's).
	// For example we receive 88 samples, if the sample per  bit value was rounded to 5 we get 17.6 -> 18 databits, 
	// but really only 16 bits were received. This makes the retiming unreliable.
	// We multiply the RX_CLK_CLOCK_RATE by 4, this way the accuracy of the calculation will be 0.25 instead of 1,
	// which enables us to handle 16 consecutive 0's or 1's.


	SamplePerBitRatio = (U16)( ( (U32)RX_CLK_Clock_Rate << 2 ) / DATARATE);

#endif


	//Initialize the MCU: 
	//	- set IO ports for the Software Development board
	//	- set MCU clock source
	//	- initialize the SPI port 
	//	- turn off LEDs
	MCU_Init();


	//enable global interrupt
	EA = 1;

	Radio_Init ();

	Deglitch_Init();

	/*MAIN Loop*/


#ifndef PACKET_HANDLER

	// if packet handler is not enabled, only the deglitching routine runs
	Deglitch_Start();

	while (1)
	{
		// the deglitching is done in the interrupt service routine
	}

#else

	// prepare to run the packet handler
	// clear the deglitching, retiming and packet handler buffer
	pointer = 0;
	fBitBufferFull = 0;
	DeglitchedBitCnt = 1;
	fSyncWordFound = 0;

	// start deglitching
	Deglitch_Start();

	while (1)
	{

		// if the bit buffer is full then call the retiming and packet handler algorithm
		if ( fBitBufferFull == TRUE )
		{
			fBitBufferFull = FALSE;
			
			// call the bit retiming algorithm
			BitsRetiming ( &DataBitBuffer[fLastBufferSelect][0], 32, SamplePerBitRatio, &RetimedBitBuffer[0], &NoOfRetimedBits);
	
			// call the packet handler								
			PacketHandler ( &RetimedBitBuffer[0], NoOfRetimedBits );

			// clear the retimed bits counter
			NoOfRetimedBits = 0;

		}

		// if a packet is received then check against the pre-defined payload
		if ( fPacketReceived == TRUE )
		{

			// stop deglitching
			Deglitch_Stop();

			fPacketReceived = FALSE;

			CheckSum = PAYLOAD_BYTE_LENGTH;
			
			// check every byte of the received data
			for ( i = 0; i < ( PAYLOAD_BYTE_LENGTH ); i++ )
			{
				if ( PayloadBuffer[i] == ComparePayload[i] )
					CheckSum--;
			}

			if ( CheckSum )
			{
				// packet received with errors				
			}

			else 
			{
				// packet received corretly
				LED2 = 1;
			}

			// some delay 
			
			for ( i = 0; i < 255; i++ )
				for ( j = 0; j < 255; j++ );

			// turn off the indicator LEDs
			LED1 = 0;
			LED2 = 0;

			// restart deglitching
			Deglitch_Start();
		}


	}

#endif

}



/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  	void MCU_Init(void)
  +
  + DESCRIPTION:   	This function configures the MCU 
  + 						P0.0  -  skipped, PB0
  + 						P0.1  -  skipped  PB1
  + 						P0.2  -  skipped, analog input, XTAL
  + 						P0.3  -  skipped, analog input, XTAL
  + 						P0.4  -  UART TX, push-pull output
  + 						P0.5  -  UART RX, open drain input
  + 						P0.6  -  skipped
  + 						P0.7  -  skipped
  +
  + 						P1.0  -  SCK  (SPI1), Push-Pull,  Digital
  + 						P1.1  -  MISO (SPI1), Open-Drain, Digital
  + 						P1.2  -  MOSI (SPI1), Push-Pull,  Digital
  + 						P1.3  -  NSS, Push-Pull,  Digital
  + 						P1.4  -  skipped LED1 
  + 						P1.5  -  skipped LED2
  + 						P1.6  -  skipped LED3
  + 						P1.7  -  skipped LED4
  +	
  + 						P2.0  -  skipped, PB3
  + 						P2.1  -  skipped  PB4
  +
  + INPUT:			  None
  +
  + RETURN:         None
  +
  + NOTES:          None
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void MCU_Init(void)
{
	//Disable the Watch Dog timer of the MCU
	PCA0MD	&= ~0x40;                  


	// clock: 24.5MHz, internal precision osc
	FLSCL	= 0x40;
	OSCICN	= 0x8F;
	CLKSEL	= 0x00;

	// Initialize the the IO ports and the cross bar
	P0MDIN	= 0xF3;
	P0MDOUT	= 0x10;
	P0SKIP	= 0xCF; 
	XBR0	= 0x01; 
	XBR1	|= 0x40;                    // Enable SPI1 (3 wire mode)
	P1MDOUT	|= 0x01;                    // Enable SCK push pull
	P1MDOUT	|= 0x04;                    // Enable MOSI push pull
	P1SKIP	|= 0x08;                    // skip NSS
	P1MDOUT	|= 0x08;                    // Enable NSS push pull
	P1SKIP	|= 0xF0;                    // skip LEDs
	P1MDOUT	|= 0xF0;                    // Enable LEDS push pull
	P2SKIP	|= 0x03;                    // skip PB3 & 4
	SFRPAGE	= CONFIG_PAGE;
	P0DRV	= 0x10;	
	P1DRV	= 0xFD;
	P2DRV	= 0xF8;    	
	SFRPAGE	= LEGACY_PAGE;
	XBR2	|= 0x40;                    // enable Crossbar

	// For the SPI communication the hardware peripheral of the MCU is used 
	//in 3 wires Single Master Mode. The select pin of the radio is controlled
	//from software
	SPI1CFG		= 0x40;					//Master SPI, CKPHA=0, CKPOL=0
	SPI1CN		= 0x00;					//3-wire Single Master, SPI enabled
	SPI1CKR		= 0x0B;					//gives 1MHz SPI clock
	SPI1EN		= 1;                   	// Enable SPI interrupt
	NSS			= 1;

	// Turn off the LEDs
	LED1 = 0;
	LED2 = 0;
	LED3 = 0;
	LED4 = 0;

}


void Deglitch_Init ( void )
{
	Radio_Init ();

#if DEGLITCH_METHOD == 1
	Deglitch1_Init();
#endif

#if DEGLITCH_METHOD == 2
	Deglitch2_Init();
#endif


}

void Deglitch_Start ( void )
{

#if DEGLITCH_METHOD == 1
	Deglitch1_Start();
#endif

#if DEGLITCH_METHOD == 2
	Deglitch2_Start();
#endif

	StartReceiver();

}

void Deglitch_Stop ( void )
{

#if DEGLITCH_METHOD == 1
	Deglitch1_Stop();
#endif

#if DEGLITCH_METHOD == 2
	Deglitch2_Stop();
#endif

	StopReceiver();

}

